PROGRAM TRAVERSE;

{$B-,D+,R-,S-,T+,V-}

USES Crt,Dos;

{  Ŀ
    Define data types                                  
   
}

TYPE

  Fptr      = ^Dir_Rec;

  Dir_Rec   = record                   { Dble ptr. record for dir. entries }
    DirChr  : char;
    DirName : string[12];
    Next    : Fptr;
    Prev    : Fptr;
  END;

  str_type  = string[12];

  panel     = array [1..4000] of byte;           { For saving screen image }

  DriveList = 'A'..'F';


{
   Ŀ
    Define Global Variables                            
   
}

VAR

  Alphabet    : string[26];           { String for alphabet characters     }
  LoopDirs    : boolean;              { Looping flag                       }
  DirName     : str_type;             { Found directory name               }
  Dir_Ptr     : Fptr;                 { Ptr. to directory name list        }
  Num_Dirs    : integer;              { Total # of dirs. found             }
  Cnt_Cur     : boolean;              { Flag for cnt'g curr. dir. or not   }
  Cur_Dir     : string[65];           { Current directory                  }
  Orig_Dir    : string[65];           { Dir. you started in                }
  Drive       : DriveList;            { Contains drive designation         }

  Row_BEGIN   : integer;              { Absolute screen Row/Col for        }
  Col_BEGIN   : integer;              { Location of Upper Left Corner      }
                                      { of dir. selection window           }
  Row_Quan    : integer;              { Number of rows                     }
  Col_Quan    : integer;              { Number of cols                     }

  Act_Attr    : integer;              { Active (highlighted) dir. vid attr }
  IAct_Attr   : integer;              { Inactive dir. video attribute      }
  Save_Attr   : integer;              { Save current text attribute        }

  Wndw_Bdr    : integer;              { Dir. selection window border type  }
  Col_Max     : integer;              { Max Col to put dir. name at        }
  Row_Max     : integer;              { Max Row to put dir. name at        }
  Cur_Col     : integer;              { Current column                     }
  Cur_Row     : integer;              { Current Row                        }
  Row_Beg     : integer;              { Beginning row of window            }
  Col_Beg     : integer;              { Beginning col of window            }

  Save_WMin   : word;                 { Save area for WindMin & WindMax    }
  Save_WMax   : word;
  Save_X      : integer;              { Cursor x,y save area               }
  Save_Y      : integer;

  HeapPtr     : pointer;              { Pointer to heap for mark/release   }

  OrigMode    : byte;                        { Original video mode         }
  Vidcolor    : panel absolute $B800:0000;   { Storage for color video mem }
  Vidmono     : panel absolute $B000:0000;   { Storage for b/w video mem   }
  Screen      : panel;

  ExitSave    : pointer;              { Pointer for exit procedure         }

{
 Ŀ
  BEGIN DownDir Procedures                           
 
}
{
   Ŀ
    PROCEDURE Beepit                                   
   
}

PROCEDURE Beepit;

BEGIN
  SOUND (760);                                          { Beep the speaker }
  DELAY (80);
  NOSOUND;
END;

{
   Ŀ
    FUNCTION VideoMode                                 
   
}

FUNCTION VideoMode : Byte;

VAR
  reg        : registers;

BEGIN
  reg.ah     := 15;                                 { Determine video type }
  INTR ($10,reg);                                   { by polling BIOS int. }
  VideoMode  := reg.al;                             { 10                   }
END;

{
   Ŀ
    FUNCTION ISColor                                   
   
}

FUNCTION ISColor : Boolean;
  BEGIN;
    IF VideoMode = 7 THEN
      ISColor  := False
    ELSE
      ISColor  := True;
  END;

{
   Ŀ
    PROCEDURE Highlight                                
   
}

PROCEDURE Highlight (ptr : Fptr);

BEGIN
  TextAttr := Act_Attr;                         { Highlight a dir. name    }
  GOTOXY (Cur_Col,Cur_Row);
  WRITE ('',ptr^.DirChr,' ',ptr^.DirName,'');
  TextAttr := IAct_Attr;
END;

{
   Ŀ
    PROCEDURE Un_Highlight                             
   
}

PROCEDURE Un_Highlight (ptr : Fptr);

BEGIN
  TextAttr := IAct_Attr;                        { UnHighlight a dir. name }
  GOTOXY (Cur_Col,Cur_Row);
  WRITE (' ',ptr^.DirChr,' ',ptr^.DirName,' ');
END;

{
   Ŀ
    PROCEDURE Save_Screen                              
   
}

PROCEDURE Save_Screen;

BEGIN
  Save_X    := WhereX;                           { Save current cursor     }
  Save_Y    := WhereY;                           { x,y coordinates         }
  Save_WMin := WindMin;                          { Save the current window }
  Save_WMax := WindMax;                          { min/max coordinates     }
  Save_Attr := TextAttr;

  IF (ISColor) THEN                              { Move screen image to    }
    Screen  := Vidcolor                          { storage depending on    }
  ELSE                                           { video card type         }
    Screen  := Vidmono;
END;

{
   Ŀ
    PROCEDURE Restore_Screen                           
   
}

PROCEDURE Restore_Screen;

BEGIN
  IF (ISColor) THEN                              { Restore original screen }
    Vidcolor := Screen                           { image                   }
  ELSE
    Vidmono  := Screen;

  TextAttr   := Save_Attr;

  WINDOW (LO(Save_WMin)+1,HI(Save_WMin)+1,
          LO(Save_WMax)+1,HI(Save_WMax)+1);
                                                 { Restore original window }
  GOTOXY (Save_X,Save_Y);                        { min/max coord.'s & cur. }

END;

{
   Ŀ
    PROCEDURE Cursor                                   
   
}

PROCEDURE Cursor (Cur_Off : Boolean);

VAR

  reg  : registers;

BEGIN
  IF (NOT Cur_Off) THEN
    BEGIN                                                { Turn cursor off }
      reg.ah := 1;
      reg.cl := 7;
      reg.ch := 32;
      INTR ($10,reg);
    END
  ELSE
    BEGIN                                                { Turn cursor on  }
      INTR ($11,reg);
      IF ((reg.al AND $10) <> 0) THEN
        reg.cx := $0B0C
      ELSE
        reg.cx := $0607;
      reg.ah := 1;
      INTR ($10,reg);
    END;
END;

{
   Ŀ
    PROCEDURE Draw_Border                              
   
}

PROCEDURE Draw_Border;

VAR

  TLC, TRC   : char;
  BLC, BRC   : char;
  HLINE      : char;
  VLINE      : char;
  TLFT, TRHT : char;
  i          : integer;

BEGIN
  CASE (Wndw_Bdr) OF                            { Define border elements   }
                                                { based on global Wndw_Bdr }
    1  :  BEGIN
            TRC   := '';
            BRC   := '';
            TLC   := '';
            BLC   := '';
            HLINE := '';
            VLINE := '';
            TLFT  := '';
            TRHT  := '';
          END;

    2  :  BEGIN
            TRC   := '';
            BRC   := '';
            TLC   := '';
            BLC   := '';
            HLINE := '';
            VLINE := '';
            TLFT  := '';
            TRHT  := '';
          END;

  END; {case}

  GOTOXY (1,1);                                 { Start to draw the border }

  WRITE (TLC);                                  { Top line with corners    }
  FOR i := 1 to (Col_Quan*16 + 2) DO WRITE (HLINE);
  WRITE (TRC);

  FOR i := 2 to Row_Quan - 1 DO                 { Vertical lines           }
    BEGIN
      GOTOXY (1,i);
      WRITE (VLINE);
      GOTOXY ((Col_Quan*16 + 4),i);
      WRITE (VLINE);
    END;

  GOTOXY (1,Row_Quan);                          { Bottom line with corners }
  WRITE (BLC);
  FOR i:=1 to (Col_Quan*16 + 2) DO WRITE (HLINE);
  WRITE (BRC);                                  { End draw of border       }

  IF ((Length(Cur_Dir) + 2) < (Col_Quan*16 + 1)) THEN
    BEGIN
      GOTOXY (2,1);                             { If dir. name fits,       }
      WRITE (TLFT,' ',Cur_Dir,' ',TRHT);        { insert it in the top row }
    END;
END;

{
   Ŀ
    PROCEDURE Make_Window                              
   
}

PROCEDURE Make_Window;

VAR

  x1,y1,x2,y2 : byte;

BEGIN
  TextAttr := IAct_Attr;                      { Define text color          }

  x1 := Col_BEGIN;                            { Define dirs. window        }
  y1 := Row_BEGIN;
  x2 := Col_BEGIN + (Col_Quan * 16) + 3;      { Width = 20 or 36 cols.     }
  y2 := Row_BEGIN + Row_Quan;
  WINDOW (x1,y1,x2,y2);                       { Activate the window        }
  ClrScr;                                     { Clear window               }

  IF (Wndw_bdr <> 0) THEN
    BEGIN
      Draw_Border;                            { Draw the window border     }
      x1 := x1 + 1;                           { Redefine window so don't   }
      x2 := x2 - 1;                           { scroll the border if there }
      y1 := y1 + 1;                           { is one.                    }
      y2 := y2 - 2;                           { Window 2 rows smaller than }
    END;                                      { border.                    }
  WINDOW (x1,y1,x2,y2);                       { Activate new window        }
  ClrScr;                                     { Clear window               }
END;

{
   Ŀ
    PROCEDURE Clr_Window                               
   
}

PROCEDURE Clr_Window;

VAR

  x1,y1,x2,y2 : byte;

BEGIN

  x1 := Col_BEGIN;                            { Define dirs. window        }
  y1 := Row_BEGIN;
  x2 := Col_BEGIN + (Col_Quan * 16) + 3;      { Width = 20 or 36 cols.     }
  y2 := Row_BEGIN + Row_Quan;

  WINDOW (x1,y1,x2,y2);                       { Activate the window        }
  ClrScr;                                     { and clear it               }

END;

{
   Ŀ
    FUNCTION Get_Dirs                                  
   
}

FUNCTION Get_Dirs (VAR First : Fptr) : integer;

VAR

  p1, p2   : Fptr;
  numdirs  : integer;
  DirInfo  : SearchRec;
  Placed   : boolean;

BEGIN
  Get_Dirs  := 0;

  FindFirst ('*.*',Directory,DirInfo);       { Find first matching dir.    }
  IF DosError = 0 THEN                       { If found a dir., continue   }
    BEGIN
      IF (((Cnt_Cur = False) AND (DirInfo.name = '.'))
      OR (DirInfo.attr <> Directory)) THEN   { Do if not count'g cur. dir. }
        BEGIN                                { or if 1st item <> dir.      }
          DirInfo.attr := 0;
          WHILE (DirInfo.attr<>Directory) DO { Look for dirs. only         }
            BEGIN
              FindNext (DirInfo);            { Find next matching dir.     }
              IF DosError <> 0 THEN HALT;    { if not count'g cur. dir.    }
            END;
        END;

      numdirs     := 1;                      { Init. count of dir.'s       }
      NEW(p1);                               { Allocate pointer to dir.    }
      First       := p1;                     { Save current directory ptr. }
      p1^.DirName := DirInfo.name;           { Copy in dir.name            }
      p1^.Prev    := nil;                    { Set up prev pointer         }
      p1^.Next    := nil;                    { Set up next pointer         }


      WHILE DosError = 0 DO                  { Get any additional dirs.    }
        BEGIN
          FindNext (DirInfo);                { Find next matching dir.     }

          IF ((DosError = 0) AND (DirInfo.attr = Directory)) THEN
                                             { If there are more, continue }
            BEGIN
              numdirs := numdirs + 1;        { Incr. number dirs. counter  }
              NEW(p1);                       { Allocate new pointer        }
              p1^.DirName := DirInfo.name;               { Copy dir. name  }
              IF (p1^.DirName < First^.DirName) THEN     { Sort dir. names }
                BEGIN
                  p1^.Next    := First;
                  First^.Prev := p1;
                  First       := p1;
                END
              ELSE
                BEGIN
                  p2     := First;
                  Placed := False;
                  WHILE ((p2^.Next <> nil) AND (Placed = False)) DO
                    BEGIN
                      IF (p1^.DirName >= p2^.Next^.DirName) THEN
                        p2 := p2^.Next
                      ELSE
                        Placed := True;
                    END;
                  p1^.Next := p2^.Next;
                  p1^.Prev := p2;
                  p2^.Next^.Prev := p1;
                  p2^.Next := p1;
                END;
            END;                                          { End sort       }
        END;
      Get_Dirs := numdirs;                  { Return number of dirs. found }
    END
END;

{
   Ŀ
    PROCEDURE Put_Dirs                                 
   
}

PROCEDURE Put_Dirs (var_ptr : Fptr; Maxdirs : integer);

VAR

  ptr1      : Fptr;
  i         : integer;
  irow,icol : integer;

BEGIN
  ptr1 := var_ptr;                              { Put the dirs. found into }
  irow := Row_Beg;                              { the dirs. window by      }
  icol := Col_Beg;                              { traversing the dir. ptr  }
                                                { linked list              }
  FOR i := 1 to Maxdirs  DO
    BEGIN
      GOTOXY (icol,irow);
      ptr1^.DirChr := Alphabet[i];
      WRITE (' ',ptr1^.DirChr,' ',ptr1^.DirName,' ');
      irow := irow + 1;
      IF (irow > Row_Max) THEN
        BEGIN
          irow := Row_Beg;
          icol := Col_Beg + 16;
        END;
      IF (ptr1^.Next <> nil) THEN
        ptr1 := ptr1^.Next
      ELSE
        i := Maxdirs;
    END;
END;

{
   Ŀ
    FUNCTION Srch_Dir                                  
   
}

FUNCTION Srch_Dir (var_ptr : Fptr; var_str : char) : Fptr;

VAR
  ptr1       :  Fptr;
  Found      :  boolean;
  str1, str2 :  char;

BEGIN
  ptr1     := var_ptr;
  Found    := False;
  str1     := var_str;
  Srch_Dir := nil;

  WHILE ((ptr1 <> nil) AND (Found = False)) DO      { Search list for list }
    BEGIN                                           { entry that matches   }
      str2 := ptr1^.DirChr;                         { keyboard entry       }
      IF str1 = str2 THEN
        BEGIN
          Srch_Dir := ptr1;
          found    := True;
        END
      ELSE
        ptr1 := ptr1^.Next;
        IF (ptr1 = nil) THEN
          BEGIN
            Srch_Dir := nil;                    { If key entry not on list }
            Beepit;                             { set Srch_Dir to nil &    }
            found    := True;                   { beep                     }
          END;
     END;
END;

{
   Ŀ
    FUNCTION Next_Dir                                  
   
}

FUNCTION Next_Dir (var_ptr : Fptr; count : integer;
                   Max_Scrn : integer) : Fptr;

VAR

  ptr1 : Fptr;
  i    : integer;

BEGIN
  ptr1 := var_ptr;
  IF (ptr1^.Next <> nil) THEN                   { Move up one dir.         }
    BEGIN                                       { Is there a next dir.?    }
      Un_Highlight(ptr1);                       { Unhighlight current dir. }
      FOR i := 1 to count DO                    { Traverse dir. list while }
        BEGIN                                   { updating the current row }
          IF (ptr1^.Next <> nil) THEN           { and col location         }
            BEGIN
              ptr1 := ptr1^.Next;
              Cur_row := Cur_row + 1;
              IF (Cur_row > Row_Max) THEN
                BEGIN
                  Cur_Row := Row_Beg;
                  Cur_Col := Cur_Col + 16;
                  IF (Cur_Col > Col_Max) THEN   { If off edge, stay put,   }
                    BEGIN                       { beep, and stop           }
                      ptr1    := ptr1^.Prev;
                      Cur_Col := Col_Beg - 16;
                      Beepit;
                      i := count;
                    END;
                END;
            END
         ELSE
            i := count;
        END;

      Highlight(ptr1);                           { All done, highlight     }
    END                                          { new current dir. name   }
  ELSE
    Beepit;
    Next_Dir := ptr1;
END;

{
   Ŀ
    FUNCTION Prev_Dir                                  
   
}

FUNCTION Prev_Dir (var_ptr : Fptr; count : integer;
                   Max_Scrn : integer) : Fptr;

VAR

  ptr1 : Fptr;
  i    : integer;

BEGIN
  ptr1 := var_ptr;
  IF (ptr1^.Prev <> nil) THEN                   { Back up one dir.         }
    BEGIN                                       { Is there a prev dir.?    }
      Un_Highlight(ptr1);                       { Unhighlight current dir. }
      FOR i := 1 to count DO                    { Traverse dir. list while }
        BEGIN                                   { updating the current row }
          IF (ptr1^.Prev <> nil) THEN           { and col location         }
            BEGIN
              ptr1 := ptr1^.Prev;
              Cur_Row := Cur_Row - 1;
              IF (Cur_Row < Row_Beg) THEN
                BEGIN
                  Cur_Row := Row_Max;
                  Cur_Col := Cur_Col - 16;
                  IF (Cur_Col < Col_Beg) THEN
                    BEGIN
                      ptr1    := ptr1^.Next;    { If off edge, stay put,   }
                      Cur_Col := Col_Beg + 16;  { beep, and stop           }
                      Beepit;
                      i := count;
                    END;
                END;
            END
          ELSE
            i := count;
        END;

      Highlight(ptr1);                          { All done, highlight      }
    END                                         { new current dir. name    }
  ELSE
    Beepit;
    Prev_Dir := ptr1;
END;

{
   Ŀ
    PROCEDURE Sel_Dir                                  
   
}

PROCEDURE Sel_Dir (VAR Dir_Name : str_type;
                   NumDirs  : integer; var_ptr : Fptr);

VAR

  ptr1, ptr2  : Fptr;
  Max_Dirs    : integer;
  Max_Scrn    : integer;
  Ch          : char;
  done        : boolean;

BEGIN
  Max_Dirs := NumDirs;                     { Init. Max_Dirs & Dir_Name     }
  Dir_Name := '';

  IF (Max_Dirs <> 0) THEN                  { Proceed if dirs. found        }
    BEGIN
      ptr1 := var_ptr;

      Col_Beg := 2;                        { Define some window limits     }
      Row_Beg := 1;
      Col_Max := (Col_Beg + ((Col_Quan - 1) * 16));

      IF (Wndw_Bdr = 0) THEN               { Compute Max rows of dirs.     }
        Row_Max := Row_Quan
      ELSE
        Row_Max := Row_Quan - 2;

      Max_Scrn := Col_Quan * Row_Max;      { Compute Max dirs. w/in window }
      IF (Max_Scrn > Max_Dirs) THEN Max_Scrn := Max_Dirs;

      Make_Window;                         { Draw the dirs. window         }

      Put_Dirs (ptr1,Max_Scrn);            { Fill window w/ avail dirs.    }

      Cur_Row := Row_Beg;                  { Initialize cur row/col        }
      Cur_Col := Col_Beg;
      Highlight (ptr1);                    { Highlight first dir.          }

      Done := False;                       { Continue till user selects a  }
      WHILE (Done = False) DO              { dir. or quits                 }
        BEGIN
          Ch := ReadKey;
          IF (Ch = #0) THEN
            BEGIN
              Ch := ReadKey;
              CASE Ch OF
                #75 : ptr1 := Prev_Dir(ptr1,13,Max_Scrn);    { Left Arrow  }
                #77 : ptr1 := Next_Dir(ptr1,13,Max_Scrn);    { Right Arrow }
                #72 : ptr1 := Prev_Dir(ptr1,1,Max_Scrn);     { Up Arrow    }
                #80 : ptr1 := Next_Dir(ptr1,1,Max_Scrn);     { Down Arrow  }

                #73: BEGIN
                       Drive    := PRED (Drive);      { If PgUp, go to     }
                       Dir_Name := Drive + ':\';      { prev. drive;       }
                       Done     := True;              { redraw scrn.       }
                      END;
                #81: BEGIN
                       Drive    := SUCC (Drive);      { If PgDn, go to     }
                       Dir_Name := Drive + ':\';      { next drive;        }
                       Done     := True;              { redraw scrn.       }
                      END;
              END;  {end case}
            END

          ELSE
            BEGIN
              CASE Ch OF

                #13 : BEGIN                           { Return Key;        }
                        Dir_Name := ptr1^.DirName;    { Return highlighted }
                        LoopDirs := False;            { dir. to caller     }
                        Done     := True;
                      END;

                #27 : BEGIN                           { Escape Key         }
                        Dir_Name := Orig_Dir;         { Retrn to orig. dir }
                        LoopDirs := False;
                        Done     := True;
                      END;

               '\','/' : BEGIN
                           Dir_Name := '\';           { If \ or /, go to   }
                           Done     := True;          { root; redraw scrn  }
                         END;                         { if looping         }

              ELSE
                BEGIN
                  Ch := UPCASE (Ch);                  { Cap letter inputs  }
                  IF ((Ch >= 'A') and (Ch <= 'Z')) THEN
                    BEGIN
                      ptr2 := ptr1;
                      ptr1 := Srch_Dir (Dir_Ptr,Ch);  { See if input is on }
                      IF ptr1 <> nil THEN             { dir. listing       }
                        BEGIN
                          Dir_Name := ptr1^.DirName;  { If so, return      }
                                                      { selected dir. name }
                          IF (Ch = 'A') AND (ptr1^.DirName = '.') THEN
                            LoopDirs := False;        { If sel. cur. dir., }
                          Done     := True;           { all done           }
                        END
                      ELSE
                        ptr1 := ptr2;                 { Else, set ptr1 to  }
                    END                               { prev. value        }
                  ELSE
                   Beepit;
                END;
              END;      {end case}

            END;
        END;

    END
  ELSE
    BEGIN
      Dir_Name := '.';                      { No dirs. found - return cur. }
    END;

END;

{
   Ŀ
    PROCEDURE Read_Parm                                
   
}

PROCEDURE Read_Parm;

VAR

  Str : string[1];

BEGIN
  Str := ParamStr(1);

  IF (Str[1] IN ['u','U']) AND (LENGTH (Cur_Dir) > 3) THEN
    BEGIN                                      { If "u" for "up", go up    }
      CHDIR ('..');                            { and stop, unless at root  }
      HALT;
    END;

  IF (Str[1] IN ['o','O']) AND (LENGTH (Cur_Dir) > 3) THEN CHDIR ('..');
                                               { If "o" for "over", go up  }
                                               { one dir., then continue & }
                                               { disp. lst, unless at root }

  IF Str[1] IN ['l','L'] THEN
    BEGIN
      LoopDirs := True;                        { Set flag if looping       }
      Cnt_Cur  := True;                        { Count cur. dir.           }
    END;

END;



{
   Ŀ
    PROCEDURE Go_Direct                                
   
}

PROCEDURE Go_Direct (Ptr1 : Fptr);

BEGIN
  WHILE (Ptr1^.Next <> nil) DO
    Ptr1 := Ptr1^.Next;

  CHDIR (Ptr1^.DirName);
  HALT;
END;

{
   Ŀ
    PROCEDURE Program Exit                             
   
}

{$F+}
PROCEDURE PgmExit;

  BEGIN
    TextMode (OrigMode);                       { Return to orig. vid. mode }
    Restore_Screen;
    Cursor (True);                             { Turn cursor back on       }
    Release (HeapPtr);                         { Restore all mem allocated }
    ExitProc := ExitSave;                      { Restore orig. exit proc.  }
  END;
{$F-}

{
   Ŀ
    BEGIN MAIN PROGRAM                                 
   
}

CONST

  Version = 'Version 1.6 -- 3-31-88';      { Version; sticks in .EXE       }

BEGIN

  ExitSave := ExitProc;                    { Save last exit proc. in chain }
  ExitProc := @PgmExit;                    { Install PgmExit in exit proc. }
                                           { chain.                        }

  Alphabet := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';    { Set up alphabet string    }

  Cnt_Cur  := False;                           { False means don't cnt cur.}
                                               { dir., True means cnt. it  }

  LoopDirs := False;                           { Init. looping flag        }

                       { START DIR. PROCESSING }

  NEW  (HeapPtr);                              { Save the current heap ptr }
  MARK (HeapPtr);

  OrigMode := VideoMode;                       { Save video mode           }

  Save_Screen;                                 { Save the current screen   }

  Cursor (False);                              { Turn off the cursor       }

  GETDIR (0,Cur_Dir);                          { Get current directory     }
  Orig_Dir := Cur_Dir;                         { Save copy of cur. dir.    }
  Drive    := Cur_Dir[1];                      { Save root designation     }

  IF ParamCount > 0 THEN Read_Parm;            { Do command line parms     }


                       {    LOOPING ROUTINE    }

  REPEAT                                       { Do routines at lease once }

    Num_Dirs := Get_Dirs (Dir_Ptr);            { Get matching dirs.        }

    IF LoopDirs = False THEN                   { Do if NOT looping         }
      CASE Num_Dirs OF                         { Test for direct movement  }
                                               { to single subdir. or to   }
        0 : Halt;                              { parent dir.               }

        1 : Go_Direct (Dir_Ptr);

        2 : IF (LENGTH (Cur_Dir) > 3) THEN Go_Direct (Dir_Ptr);

        3 : IF ((LENGTH (Cur_Dir) > 3) AND (Cnt_Cur = True)) THEN
            Go_Direct (Dir_Ptr);

      END;  {end case}

             { Windowed list of directories procedures and routines follows}

                                               { Auto. sizing of window    }
    IF (Num_Dirs < 14) AND (LoopDirs = False) THEN
      BEGIN                                    { Calc. small window size   }
        Row_Quan  := Num_Dirs + 2;
        Col_Quan  := 1;
        Col_BEGIN := 60;                       { Col to place window       }
      END
    ELSE
      BEGIN                                    { Calc. large window size   }
        IF (Num_Dirs > 26) OR (LoopDirs = True) THEN
          Row_Quan := 13 + 2                   { 1/2 dir. entries + 2 for  }
        ELSE                                   { border                    }
          Row_Quan := ROUND((Num_Dirs/2)+2);   { Balance dirs. betw 2 cols }
        Col_Quan  := 2;
        Col_BEGIN := 44;                       { Col to place window       }
      END;

    Row_BEGIN   := 10;                         { Row to place window       }
    Act_Attr    := 112;                        { Normal color of window    }
    IAct_Attr   := 31;                         { Highlight color           }
    Wndw_Bdr    := 1;                          { Deflt border = sngle line }
                                               { Border types are:
                                                   0  = No Border
                                                   1  = Single line
                                                   2  = Double Line        }

    ClrScr;                                    { Use if want clr. scrn     }
                                               { while disp'g dir. list    }

    Sel_Dir (DirName,Num_Dirs,Dir_Ptr);        { Select directory          }

    Clr_Window;                                { Clr win after disp.       }

    CHDIR (DirName);                           { Go to selected dir.       }

    GETDIR (0,Cur_Dir)                         { Get current directory     }

  UNTIL LoopDirs = False;                      { Looping test              }

END.
